@@ -572,8 +572,7 @@ changed_common(
 		    && (last < wp->w_topline
 			|| (wp->w_topline >= lnum
 			    && wp->w_topline < lnume
-			    && win_linetabsize(wp, wp->w_topline,
-					ml_get(wp->w_topline), (colnr_T)MAXCOL)
+			    && linetabsize_eol(wp, wp->w_topline)
 				    <= wp->w_skipcol + sms_marker_overlap(wp, -1))))
 		wp->w_skipcol = 0;
 
@@ -1355,17 +1354,17 @@ del_bytes(
     mch_memmove(newp + col, oldp + col + count, (size_t)movelen);
     if (alloc_newp)
 	ml_replace(lnum, newp, FALSE);
-#ifdef FEAT_PROP_POPUP
     else
     {
+#ifdef FEAT_PROP_POPUP
 	// Also move any following text properties.
 	if (oldlen + 1 < curbuf->b_ml.ml_line_len)
 	    mch_memmove(newp + newlen + 1, oldp + oldlen + 1,
 			       (size_t)curbuf->b_ml.ml_line_len - oldlen - 1);
+#endif
 	curbuf->b_ml.ml_line_len -= count;
 	curbuf->b_ml.ml_line_textlen = 0;
     }
-#endif
 
     // mark the buffer as changed and prepare for displaying
     inserted_bytes(lnum, col, -count);
